/*
 * Gui.hpp
 * Copyright (C) Tobias Groenhagen 2009
 *
 */


#ifndef __GUI_H__
#define __GUI_H__
#include <QApplication>
#include <QDialog>
#include <vector>
#include "ui_qt4ps.h"
#include "ui_qt4psinfo.h"
#include "proc.hpp"

class ProcessTreeModel : public QAbstractListModel
{
  Q_OBJECT
  public:
    ProcessTreeModel(ProcessList *proc, QObject *parent=0);
    QModelIndex index(
                       int pos, int column,
                       const QModelIndex &parent=QModelIndex()
                     ) const;
    int rowCount(const QModelIndex &parent = QModelIndex()) const;
    int columnCount(const QModelIndex &parent = QModelIndex()) const;
    QModelIndex parent(const QModelIndex &child) const;
    QVariant data(const QModelIndex &index, int role) const;
    QVariant headerData(
                         int section,
                         Qt::Orientation orientation,
                         int role = Qt::DisplayRole
                       ) const;
    bool hasChildren(const QModelIndex & parent = QModelIndex()) const;

  private:
    ProcessList *proc;
    QModelIndex ProcessToIndex(
                                const Process *p, int colomn
                              ) const;
    Process* IndexToProcess(const QModelIndex &index) const;
    mutable QMap<QModelIndex,QModelIndex> parentHash;
};

class ProcessListModel : public QAbstractListModel
{
  Q_OBJECT
  public:
    ProcessListModel(ProcessList *proc, QObject *parent=0);
    int rowCount(const QModelIndex &parent = QModelIndex()) const;
    int columnCount(const QModelIndex &parent = QModelIndex()) const;
    QVariant data(const QModelIndex &index, int role) const;
    QVariant headerData(
                         int section,
                         Qt::Orientation orientation,
                         int role = Qt::DisplayRole
                       ) const;
  private:
    ProcessList *proc;
};

class Info : public QDialog, private Ui_infoWin
{
  Q_OBJECT
  public:
    Info(QWidget *parent = 0);
    void setProcess(const Process *p);
  private slots:
    void clean(void);
};

class Window : public QDialog, private Ui_mainWin
{
  Q_OBJECT
  public:
    Window(std::string path, Info *iwin, QWidget *parent = 0);
    ~Window();
  private slots:
    void ende(void);
    void showTreeInfo(const QModelIndex &index);
    void showListInfo(const QModelIndex &index);
  private:
    ProcessListModel *listm;
    ProcessTreeModel *treem;
    Info *iwin;
    ProcessList *proc;
};

class Gui
{
  public:
    Gui(std::string proc,int argc, char *argv[]);
    ~Gui();
    int show();
  private:
    QApplication *app;
    Window *win;
    Info *winf;
};

#endif
