/*
 * process_list.hpp
 * Copyright (C) Tobias Groenhagen 2009
 *
 */

#ifndef __PROC_H__
#define __PROC_H__
#include <string>
#include <vector>

class Process
{
  public:
    Process(std::string path="/proc", int pid=0);
    std::string   getCMD() const;
    int           getPID() const;
    int           getPPID() const;
    std::string   getName() const;
    char          getState() const;
    int           getTTY() const;
    unsigned long getUTime() const;
    unsigned long getSTime() const;
    unsigned long getCUTime() const;
    unsigned long getCSTime() const;
    long          getNice() const;
    unsigned long getStartTime() const;

    std::string   getLine() const;
    bool          isRoot() const;

    bool operator< (const Process& p) const;
  protected:
    std::string path;
    std::string cmd;
    std::string proc;

    int           pid;
    std::string   name;
    char          state;
    int           ppid;
    int           pgrp;
    int           session;
    int           tty_nr;
    int           tty_grp;
    unsigned long flags;
    unsigned long min_flt;
    unsigned long cmin_flt;
    unsigned long maj_flt;
    unsigned long cmaj_flt;
    unsigned long utime;
    unsigned long stime;
    long          cutime;
    long          cstime;
    long          priority;
    long          nice;
    int           num_threads;
    int           unused1;
    unsigned long start_time;
    unsigned long vsize;
    long          rss;
    unsigned long rsslim;
    unsigned long start_code;
    unsigned long end_code;
    unsigned long start_stack;
    unsigned long esp;
    unsigned long eip;
    unsigned long pending;
    unsigned long blocked;
    unsigned long sigign;
    unsigned long sigcatch;
    unsigned long wchan;
    unsigned long unused2;
    unsigned long unused3;
    int           exit_signal;
    int           task_cpu;
    unsigned long rt_priority;
    unsigned long policy;

    bool root;

    void read_cmd();
    void read_stat();
    void read_kernel_stat();
    void init();
};


class ProcessList
{
  public:
    ProcessList(std::string path="/proc");
    ~ProcessList();

    std::vector <int>           getPIDs(void);
    std::vector<Process>* getProcesses(void);

    Process*              getProcess(int pid);

    Process*              getParentProcess(int pid);
    Process*              getParentProcess(Process *p);
    int                         getParentPID(int pid);

    std::vector<Process*> getChildProcesses(int ppid);
    Process*              getChildProcessByPos(int ppid, unsigned int pos);
    Process*              getChildProcessByPID(int ppid, int pid);

    std::vector<int>            getChildPIDs(int pid);
    unsigned int                getChildPosByPID(int ppid, int pid);
    int                         getChildPIDByPos(int ppid, unsigned int pos);

    unsigned int                getChildProcessLength(int pid);

  protected:
    std::string path;
  private:
    std::vector<Process> *list;
    void readProcesses();
    std::vector <int> readPIDs();
};

#endif
